@@ -1,6 +1,6 @@
 #
 # Alexandria3k Crossref bibliographic metadata processing
-# Copyright (C) 2022  Diomidis Spinellis
+# Copyright (C) 2022-2023  Diomidis Spinellis
 # SPDX-License-Identifier: GPL-3.0-or-later
 #
 # This program is free software: you can redistribute it and/or modify
@@ -26,11 +26,11 @@ import sqlite3
 # pylint: disable-next=import-error
 import apsw
 
-from .common import add_columns, fail, log_sql, set_fast_writing
-from . import debug
-from . import perf
-from .tsort import tsort
-from .virtual_db import (
+from alexandria3k.common import add_columns, fail, log_sql, set_fast_writing
+from alexandria3k import debug
+from alexandria3k import perf
+from alexandria3k.tsort import tsort
+from alexandria3k.virtual_db import (
     ColumnMeta,
     TableMeta,
     CONTAINER_ID_COLUMN,
@@ -876,9 +876,25 @@ class IndexManager:
 
 
 class Crossref:
-    """Create a Crossref meta-data object that support queries over its
-    (virtual) table and the population of an SQLite database with its
-    data"""
+    """
+    Create a Crossref meta-data object that supports queries over its
+    (virtual) tables and the population of an SQLite database with its
+    data.
+
+    :param crossref_directory: The directory path where the Crossref
+        data files are located
+    :type crossref_directory: str
+    :param sample: A callable to control container sampling, defaults
+        to `lambda n: True`.
+        The population or query method will call this argument
+        for each Crossref container file with each container's file
+        name as its argument.  When the callable returns `True` the
+        container file will get processed, when it returns `False` the
+        container will get skipped.
+    :type sample: callable, optional
+    """
+
+    # pylint: disable=too-many-instance-attributes
 
     def __init__(self, crossref_directory, sample=lambda n: True):
         # A named in-memory database; it can be attached by name to others
@@ -899,6 +915,7 @@ class Crossref:
         self.population_columns = {}
         self.query_and_population_columns = {}
         self.index_manager = None
+        self.attached_databases = []
 
         for table in tables:
             self.vdb.execute(
@@ -929,9 +946,13 @@ class Crossref:
             self.query_columns.
             See https://rogerbinns.github.io/apsw/tips.html#parsing-sql"""
 
-            def authorizer(op_code, table, column, _database, _trigger):
+            def authorizer(op_code, table, column, database, _trigger):
                 """Query authorizer to monitor used columns"""
-                if op_code == apsw.SQLITE_READ and column:
+                if (
+                    op_code == apsw.SQLITE_READ
+                    and column
+                    and database not in self.attached_databases
+                ):
                     # print(f"AUTH: adding {table}.{column}")
                     Crossref.add_column(self.query_columns, table, column)
                 return apsw.SQLITE_OK
@@ -957,19 +978,31 @@ class Crossref:
         self.cursor.setexectrace(None)
 
     def query(self, query, partition=False):
-        """Run the specified query on the virtual database.
-        Returns an iterable over the query's results.
-        Queries involving table joins will run substantially faster
-        if access to each table's records is restricted with
-        an expression `table_name.container_id = CONTAINER_ID`, and
-        the `partition` argument is set to true.
-        In such a case the query is repeatedly run over each database
-        partition (compressed JSON file) with `CONTAINER_ID` iterating
-        sequentially to cover all partitions.
-        The query's result is the concatenation of the individal partition
-        results.
-        Running queries with joins without partitioning will often result
-        in quadratic (or worse) algorithmic complexity."""
+        """
+        Run the specified query on the virtual database using the data
+        specified in the object constructor's call.
+
+        :param query: An SQL `SELECT` query specifying the required data.
+        :type query: str
+
+        :param partition: When true the query will run separately in each
+            container, defaults to `False`.
+            Queries involving table joins will run substantially faster
+            if access to each table's records is restricted with
+            an expression `table_name.container_id = CONTAINER_ID`, and
+            the `partition` argument is set to true.
+            In such a case the query is repeatedly run over each database
+            partition (compressed JSON file) with `CONTAINER_ID` iterating
+            sequentially to cover all partitions.
+            The query's result is the concatenation of the individal partition
+            results.
+            Running queries with joins without partitioning will often result
+            in quadratic (or worse) algorithmic complexity.
+        :type partition: bool, optional
+
+        :return: An iterable over the query's results.
+        :rtype: iterable
+        """
 
         self.cursor = self.vdb.cursor()
 
@@ -1026,32 +1059,45 @@ class Crossref:
         """Return the column names associated with an executing query"""
         return [description[0] for description in self.cursor.description]
 
-    def populate(self, database_path, columns=None, condition=None):
-        """Populate the specified SQLite database.
+    def populate(
+        self,
+        database_path,
+        columns=None,
+        condition=None,
+        attach_databases=None,
+    ):
+        """
+        Populate the specified SQLite database using the data specified
+        in the object constructor's call.
         The database is created if it does not exist.
-        If it exists, the populated tables are dropped
+        If it exists, the tables to be populated are dropped
         (if they exist) and recreated anew as specified.
 
-        columns is an array containing strings of
-        table_name.column_name or table_name.*
+        :param database_path: The path specifying the SQLite database
+            to populate.
+        :type database_path: str
 
-        conditions is a dictionary of table_name to condition
+        :param columns: A list of strings specifying the columns to
+            populate, defaults to `None`.  The strings are of the form
+            `table_name.column_name` or `table_name.*`.
+        :type columns: list, optional
 
-        The condition is an
-        [SQL expression](https://www.sqlite.org/syntax/expr.html)
-        containing references to the table's columns.
-        It can also contain references to populated tables, by prefixing
-        the column name with `populated.`.
-        Implicitly, if a main table is populated, its detail tables
-        will only get populated with the records associated with the
-        correspoing main table.
+        :param condition: `SQL expression`_ specifying the rows to include
+            in the database's population, defaults to `None`.
+            The expression can contain references to the table's columns.
+            Implicitly, if a main table is populated, its details tables
+            will only get populated with the records associated with the
+            correspoing main table's record.
+        :type condition: str, optional
 
-        indexes is an array of table_name(indexed_column...)  strings,
-        that specifies indexes to be created before populating the tables.
-        The indexes can be used to speed up the evaluation of the population
-        conditions.
-        Note that foreign key indexes will always be created and need
-        not be specified.
+        :param attach_databases: A list of colon-joined tuples specifying
+            a database name and its path, defaults to `None`.
+            The specified databases are attached and made available to the
+            row selection query through the specified database name.
+        :type attach_databases: list, optional
+
+
+        .. _SQL expression: https://www.sqlite.org/syntax/expr.html
         """
 
         # pylint: disable=too-many-statements
@@ -1120,6 +1166,27 @@ class Crossref:
                     )
             return result
 
+        def populate_only_root_table(table, partition_index, condition):
+            """Populate the root table, when no other tables will be needed"""
+
+            columns = ", ".join(
+                [f"{table}.{col}" for col in self.population_columns[table]]
+            )
+            if not condition:
+                condition = "true"
+            # No need for temp table matching at the root
+            self.vdb.execute(
+                log_sql(
+                    f"""
+                INSERT INTO populated.{table}
+                SELECT {columns} FROM {table}
+                WHERE {table}.container_id = {partition_index}
+                  AND {condition}
+            """
+                )
+            )
+            perf.log(f"Populate {table}")
+
         def populate_table(table, partition_index, condition):
             """Populate the specified table"""
 
@@ -1154,7 +1221,7 @@ class Crossref:
             self.vdb.execute(log_sql(statement))
             perf.log(f"Populate {table}")
 
-        def create_database_schema(columns):
+        def create_database_schema(columns, attach_databases):
             """Create the populated database, if needed"""
             if not os.path.exists(database_path):
                 pdb = sqlite3.connect(database_path)
@@ -1165,6 +1232,18 @@ class Crossref:
             )
             set_fast_writing(self.vdb)
 
+            for db_spec in attach_databases:
+                try:
+                    (db_name, db_path) = db_spec.split(":")
+                except ValueError:
+                    fail(
+                        f"Invalid database specification: '{db_spec}'; expected name:path"
+                    )
+                self.vdb.execute(
+                    log_sql(f"ATTACH DATABASE '{db_path}' AS {db_name}")
+                )
+                self.attached_databases.append(db_name)
+
             self.index_manager = IndexManager(self.vdb)
 
             add_columns(
@@ -1194,13 +1273,9 @@ class Crossref:
                 )
             perf.log("Table creation")
 
-        def create_matched_tables():
+        def create_matched_tables(matched_tables):
             """Create copies of the virtual tables for fast access"""
-            query_table_names = tables_transitive_closure(
-                self.query_columns.keys(), "works"
-            )
-
-            for table in query_and_population_tables():
+            for table in matched_tables:
                 columns = self.query_and_population_columns.get(table)
                 if columns:
                     columns = set.union(columns, {"rowid"})
@@ -1225,6 +1300,9 @@ class Crossref:
             # Create a table containing the work ids for all works
             # matching the query, which is executed in a context
             # containing all required tables.
+            query_table_names = tables_transitive_closure(
+                self.query_columns.keys(), "works"
+            )
             create = (
                 """CREATE TEMP TABLE temp_matched AS
                         SELECT works.id, works.rowid
@@ -1242,23 +1320,30 @@ class Crossref:
 
             perf.log("Matched table creation")
 
-        create_database_schema(columns)
+        if attach_databases is None:
+            attach_databases = []
+        create_database_schema(columns, attach_databases)
         # Populate all tables from the records of each file in sequence.
         # This improves the locality of reference and through the constraint
         # indexing and the file cache avoids opening, reading, decompressing,
         # and parsing each file multiple times.
+        matched_tables = query_and_population_tables()
         for i in self.data_source.get_file_id_iterator():
             debug.log(
                 "progress",
                 f"Container {i} {self.data_source.get_file_name_by_id(i)}",
             )
 
-            if condition:
-                create_matched_tables()
+            if len(matched_tables) == 1:
+                (table,) = self.population_columns
+                populate_only_root_table(table, i, condition)
+            else:
+                if condition:
+                    create_matched_tables(matched_tables)
 
-            for table in self.population_columns:
-                populate_table(table, i, condition)
-            self.index_manager.drop_indexes()
+                for table in self.population_columns:
+                    populate_table(table, i, condition)
+                self.index_manager.drop_indexes()
         perf.log("Table population")
 
         self.vdb.execute(log_sql("DETACH populated"))
